@@ -7,10 +7,9 @@ define([
     'base/js/i18n',
     'base/js/security',
     'base/js/keyboard',
+    'base/js/markdown',
     'services/config',
-    'notebook/js/mathjaxutils',
-    'components/marked/lib/marked',
-], function($, utils, i18n, security, keyboard, configmod, mathjaxutils, marked) {
+], function($, utils, i18n, security, keyboard, markdown, configmod) {
     "use strict";
 
     /**
@@ -49,7 +48,7 @@ define([
     OutputArea.config_defaults = {
         stream_chunk_size: 8192, // chunk size for stream output
     };
-    
+
     /**
      * Class prototypes
      **/
@@ -80,17 +79,17 @@ define([
         if (!this.prompt_area) {
             this.prompt_overlay.hide();
         }
-        
+
         this.wrapper.addClass('output_wrapper');
         this.element.addClass('output');
-        
+
         this.collapse_button.addClass("btn btn-default output_collapsed");
         this.collapse_button.attr('title', i18n.msg._('click to expand output'));
         this.collapse_button.text('. . .');
-        
+
         this.prompt_overlay.addClass('out_prompt_overlay prompt');
         this.prompt_overlay.attr('title', i18n.msg._('click to expand output; double click to hide output'));
-        
+
         this.expand();
     };
 
@@ -114,7 +113,7 @@ define([
         if (threshold <=0) {
             return false;
         }
-        // line-height from http://stackoverflow.com/questions/1185151
+        // line-height from https://stackoverflow.com/questions/1185151
         var fontSize = this.element.css('font-size') || '14px';
         var lineHeight = Math.floor((parseFloat(fontSize.replace('px','')) || 14) * 1.3);
         return (this.element.height() > threshold * lineHeight);
@@ -256,7 +255,7 @@ define([
         }
         this.append_output(json);
     };
-    
+
     // Declare mime type as constants
     var MIME_JAVASCRIPT = 'application/javascript';
     var MIME_HTML = 'text/html';
@@ -268,8 +267,8 @@ define([
     var MIME_GIF = 'image/gif';
     var MIME_PDF = 'application/pdf';
     var MIME_TEXT = 'text/plain';
-    
-    
+
+
     OutputArea.output_types = [
         MIME_JAVASCRIPT,
         MIME_HTML,
@@ -305,10 +304,10 @@ define([
         });
         return bundle;
     };
-    
+
     OutputArea.prototype.append_output = function (json) {
         this.expand();
-        
+
         if (this.clear_queued) {
             this.clear_output(false);
             this._needs_height_reset = true;
@@ -370,6 +369,7 @@ define([
     OutputArea.prototype.create_output_area = function () {
         var oa = $("<div/>").addClass("output_area");
         if (this.prompt_area) {
+            oa.append($('<div/>').addClass('run_this_cell'));
             oa.append($('<div/>').addClass('prompt'));
         }
         return oa;
@@ -388,6 +388,8 @@ define([
 
     OutputArea.prototype.create_output_subarea = function(md, classes, mime) {
         var subarea = $('<div/>').addClass('output_subarea').addClass(classes);
+        // Unforce RTL
+        subarea.attr("dir","auto");
         if (_get_metadata_key(md, 'isolated', mime)) {
             // Create an iframe to isolate the subarea from the rest of the
             // document
@@ -399,7 +401,7 @@ define([
             // Once the iframe is loaded, the subarea is dynamically inserted
             iframe.on('load', function() {
                 // Workaround needed by Firefox, to properly render svg inside
-                // iframes, see http://stackoverflow.com/questions/10177190/
+                // iframes, see https://stackoverflow.com/questions/10177190/
                 // svg-dynamically-added-to-iframe-does-not-render-correctly
                 this.contentDocument.open();
 
@@ -441,7 +443,7 @@ define([
             .append($('<div/>').text(err.toString()).addClass('js-error'))
             .append($('<div/>').text(i18n.msg._('See your browser Javascript console for more details.')).addClass('js-error'));
     };
-    
+
     OutputArea.prototype._safe_append = function (toinsert, toreplace) {
         /**
          * safely append an item to the document
@@ -461,6 +463,8 @@ define([
             // the prompt area and the proper indentation.
             toinsert = this.create_output_area();
             var subarea = $('<div/>').addClass('output_subarea');
+            // Unforce RTL
+            subarea.attr("dir","auto");
             toinsert.append(subarea);
             this._append_javascript_error(err, subarea);
             this.element.append(toinsert);
@@ -471,7 +475,7 @@ define([
     };
 
     OutputArea.output_prompt_classical = function(prompt_value) {
-        return $('<bdi>').text(i18n.msg.sprintf(i18n.msg._('Out[%d]:'),prompt_value));
+        return $('<bdi>').text(i18n.msg.sprintf(i18n.msg._('Out[%s]:'),prompt_value));
     };
 
     OutputArea.output_prompt_function = OutputArea.output_prompt_classical;
@@ -588,6 +592,8 @@ define([
         var that = this;
         var toinsert = this.create_output_area();
         var subarea = $('<div/>').addClass('output_subarea output_unrecognized');
+        // Unforce RTL
+        subarea.attr("dir","auto");
         toinsert.append(subarea);
         subarea.append(
             $("<a>")
@@ -631,6 +637,10 @@ define([
             (json.data[MIME_MARKDOWN] !== undefined)) {
             this.typeset();
         }
+        this.events.trigger('output_updated.OutputArea', {
+            output: json,
+            output_area: this,
+        });
     };
 
     OutputArea.prototype._record_display_id = function (json, element) {
@@ -647,7 +657,7 @@ define([
             element: element,
         });
     };
-    
+
     OutputArea.prototype.append_display_data = function (json, handle_inserted) {
         var toinsert = this.create_output_area();
         this._record_display_id(json, toinsert);
@@ -674,21 +684,31 @@ define([
             var type = OutputArea.display_order[i];
             var append = OutputArea.append_map[type];
             if ((json.data[type] !== undefined) && append) {
-                var value = json.data[type];
-                if (!this.trusted && !OutputArea.safe_outputs[type]) {
-                    // not trusted, sanitize HTML
-                    if (type===MIME_HTML || type==='text/svg') {
-                        value = security.sanitize_html(value);
-                    } else {
-                        // don't display if we don't know how to sanitize it
-                        console.log("Ignoring untrusted " + type + " output.");
-                        continue;
-                    }
-                }
                 var md = json.metadata || {};
-                var toinsert = append.apply(this, [value, md, element, handle_inserted]);
+                var value = json.data[type];
+                var toinsert;
+
+                try {
+                    if (!this.trusted && !OutputArea.safe_outputs[type]) {
+                        // not trusted, sanitize HTML
+                        if (type===MIME_HTML || type==='text/svg') {
+                            var parsed = $(security.sanitize_html_and_parse(value));
+                            toinsert = append.apply(this, [parsed, md, element, handle_inserted]);
+                        } else {
+                            // don't display if we don't know how to sanitize it
+                            console.log("Ignoring untrusted " + type + " output.");
+                            continue;
+                        }
+                    } else {
+                        toinsert = append.apply(this, [value, md, element, handle_inserted]);
+                    }
+                } catch (e) {
+                    console.error('Failed to render mimetype "' + type + '" with: ', e);
+                    continue;
+                }
+
                 // Since only the png and jpeg mime types call the inserted
-                // callback, if the mime type is something other we must call the 
+                // callback, if the mime type is something other we must call the
                 // inserted callback only when the element is actually inserted
                 // into the DOM.  Use a timeout of 0 to do this.
                 if ([MIME_PNG, MIME_JPEG, MIME_GIF].indexOf(type) < 0 && handle_inserted !== undefined) {
@@ -698,6 +718,7 @@ define([
                 return toinsert;
             }
         }
+
         return null;
     };
 
@@ -713,22 +734,13 @@ define([
     };
 
 
-    var append_markdown = function(markdown, md, element) {
+    var append_markdown = function(text, md, element) {
         var type = MIME_MARKDOWN;
         var toinsert = this.create_output_subarea(md, "output_markdown rendered_html", type);
-        var text_and_math = mathjaxutils.remove_math(markdown);
-        var text = text_and_math[0];
-        var math = text_and_math[1];
-        // Prevent marked from returning inline styles for table cells
-        var renderer = new marked.Renderer();
-        renderer.tablecell = function (content, flags) {
-          var type = flags.header ? 'th' : 'td';
-          var start_tag = '<' + type + '>';
-          var end_tag = '</' + type + '>\n';
-          return start_tag + content + end_tag;
-        };
-        marked(text, { renderer: renderer }, function (err, html) {
-            html = mathjaxutils.replace_math(html, math);
+        markdown.render(text, {
+            with_math: true,
+            clean_tables: true
+        }, function (err, html) {
             toinsert.append(html);
         });
         dblclick_to_reset_size(toinsert.find('img'));
@@ -778,7 +790,7 @@ define([
         var type = MIME_SVG;
         var toinsert = this.create_output_subarea(md, "output_svg", type);
 
-        // Get the svg element from within the HTML. 
+        // Get the svg element from within the HTML.
         // One svg is supposed, but could embed other nested svgs
         var svg = $($('<div \>').html(svg_html).find('svg')[0]);
         var svg_area = $('<div />');
@@ -814,7 +826,7 @@ define([
             }
         });
     }
-    
+
     var set_width_height = function (img, md, mime) {
         /**
          * set width and height of an img element from metadata
@@ -827,17 +839,17 @@ define([
             img.addClass('unconfined');
         }
     };
-    
-    var append_png = function (png, md, element, handle_inserted) {
-        var type = MIME_PNG;
-        var toinsert = this.create_output_subarea(md, "output_png", type);
+
+    OutputArea.prototype._append_img = function (src_type, md, element, handle_inserted, MIME, type_string) {
+        var type = MIME;
+        var toinsert = this.create_output_subarea(md, 'output_' + type_string, type);
         var img = $("<img/>");
         if (handle_inserted !== undefined) {
             img.on('load', function(){
                 handle_inserted(img);
             });
         }
-        img[0].src = 'data:image/png;base64,'+ png;
+        img[0].src = 'data:image/' + type_string + ';base64,'+ src_type;
         set_width_height(img, md, type);
         dblclick_to_reset_size(img);
         toinsert.append(img);
@@ -845,41 +857,17 @@ define([
         return toinsert;
     };
 
+    var append_png = function (png, md, element, handle_inserted) {
+        return this._append_img(png, md, element, handle_inserted, MIME_PNG, 'png');
+    };
 
     var append_jpeg = function (jpeg, md, element, handle_inserted) {
-        var type = MIME_JPEG;
-        var toinsert = this.create_output_subarea(md, "output_jpeg", type);
-        var img = $("<img/>");
-        if (handle_inserted !== undefined) {
-            img.on('load', function(){
-                handle_inserted(img);
-            });
-        }
-        img[0].src = 'data:image/jpeg;base64,'+ jpeg;
-        set_width_height(img, md, type);
-        dblclick_to_reset_size(img);
-        toinsert.append(img);
-        element.append(toinsert);
-        return toinsert;
-    };
-    
-    var append_gif = function (gif, md, element, handle_inserted) {
-        var type = MIME_GIF;
-        var toinsert = this.create_output_subarea(md, "output_gif", type);
-        var img = $("<img/>");
-        if (handle_inserted !== undefined) {
-            img.on('load', function(){
-                handle_inserted(img);
-            });
-        }
-        img[0].src = 'data:image/gif;base64,'+ gif;
-        set_width_height(img, md, type);
-        dblclick_to_reset_size(img);
-        toinsert.append(img);
-        element.append(toinsert);
-        return toinsert;
+        return this._append_img(jpeg, md, element, handle_inserted, MIME_JPEG, 'jpeg');
     };
 
+    var append_gif = function (gif, md, element, handle_inserted) {
+        return this._append_img(gif, md, element, handle_inserted, MIME_GIF, 'gif');
+    };
 
     var append_pdf = function (pdf, md, element) {
         var type = MIME_PDF;
@@ -903,18 +891,18 @@ define([
         element.append(toinsert);
         return toinsert;
     };
-    
+
     OutputArea.prototype.append_raw_input = function (msg) {
         var that = this;
         this.expand();
         var content = msg.content;
         var area = this.create_output_area();
-        
+
         // disable any other raw_inputs, if they are left around
         $("div.output_subarea.raw_input_container").remove();
-        
+
         var input_type = content.password ? 'password' : 'text';
-        
+
         area.append(
             $("<div/>")
             .addClass("box-flex1 output_subarea raw_input_container")
@@ -937,8 +925,9 @@ define([
                     })
                 )
             )
+            .attr("dir","auto")
         );
-        
+
         this.element.append(area);
         var raw_input = area.find('input.raw_input');
         // Register events that enable/disable the keyboard manager while raw
@@ -1004,7 +993,7 @@ define([
                 this.element.height(height);
                 this.clear_queued = false;
             }
-            
+
             // Clear all
             // Remove load event handlers from img tags because we don't want
             // them to fire if the image is never added to the page.
@@ -1015,7 +1004,7 @@ define([
             // Notify others of changes.
             this.element.trigger('changed', {output_area: this});
             this.element.trigger('cleared', {output_area: this});
-            
+
             this.outputs = [];
             this._display_id_targets = {};
             this.trusted = true;
@@ -1122,11 +1111,11 @@ define([
     OutputArea.append_map[MIME_LATEX] = append_latex;
     OutputArea.append_map[MIME_JAVASCRIPT] = append_javascript;
     OutputArea.append_map[MIME_PDF] = append_pdf;
-    
+
     OutputArea.prototype.mime_types = function () {
         return OutputArea.display_order;
     };
-    
+
     OutputArea.prototype.register_mime_type = function (mimetype, append, options) {
         if (mimetype && typeof(append) === 'function') {
             OutputArea.output_types.push(mimetype);
