@@ -1,4 +1,3 @@
-# coding: utf-8
 """Test installation of notebook extensions"""
 
 # Copyright (c) Jupyter Development Team.
@@ -6,6 +5,7 @@
 
 import glob
 import os
+import pytest
 import sys
 import tarfile
 import zipfile
@@ -14,12 +14,8 @@ from os.path import basename, join as pjoin
 from traitlets.tests.utils import check_help_all_output
 from unittest import TestCase
 
-try:
-    from unittest.mock import patch
-except ImportError:
-    from mock import patch # py2
+from unittest.mock import patch
 
-import ipython_genutils.testing.decorators as dec
 from ipython_genutils import py3compat
 from ipython_genutils.tempdir import TemporaryDirectory
 from notebook import nbextensions
@@ -35,7 +31,7 @@ from notebook.config_manager import BaseJSONConfigManager
 
 def touch(file_name, mtime=None):
     """ensure a file exists, and set its modification time
-    
+
     returns the modification time of the file
     """
     open(file_name, 'a').close()
@@ -55,7 +51,7 @@ def test_help_output():
 
 
 class TestInstallNBExtension(TestCase):
-    
+
     def tempdir(self):
         td = TemporaryDirectory()
         self.tempdirs.append(td)
@@ -73,9 +69,9 @@ class TestInstallNBExtension(TestCase):
 
         self.src = self.tempdir()
         self.files = files = [
-            pjoin(u'ƒile'),
-            pjoin(u'∂ir', u'ƒile1'),
-            pjoin(u'∂ir', u'∂ir2', u'ƒile2'),
+            pjoin('ƒile'),
+            pjoin('∂ir', 'ƒile1'),
+            pjoin('∂ir', '∂ir2', 'ƒile2'),
         ]
         for file_name in files:
             fullpath = os.path.join(self.src, file_name)
@@ -111,30 +107,30 @@ class TestInstallNBExtension(TestCase):
     def assert_dir_exists(self, path):
         if not os.path.exists(path):
             do_exist = os.listdir(os.path.dirname(path))
-            self.fail(u"%s should exist (found %s)" % (path, do_exist))
-    
+            self.fail(f"{path} should exist (found {do_exist})")
+
     def assert_not_dir_exists(self, path):
         if os.path.exists(path):
-            self.fail(u"%s should not exist" % path)
-    
+            self.fail(f"{path} should not exist")
+
     def assert_installed(self, relative_path, user=False):
         if user:
-            nbext = pjoin(self.data_dir, u'nbextensions')
+            nbext = pjoin(self.data_dir, 'nbextensions')
         else:
             nbext = self.system_nbext
         self.assert_dir_exists(
             pjoin(nbext, relative_path)
         )
-    
+
     def assert_not_installed(self, relative_path, user=False):
         if user:
-            nbext = pjoin(self.data_dir, u'nbextensions')
+            nbext = pjoin(self.data_dir, 'nbextensions')
         else:
             nbext = self.system_nbext
         self.assert_not_dir_exists(
             pjoin(nbext, relative_path)
         )
-    
+
     def test_create_data_dir(self):
         """install_nbextension when data_dir doesn't exist"""
         with TemporaryDirectory() as td:
@@ -149,61 +145,61 @@ class TestInstallNBExtension(TestCase):
                         pjoin(basename(self.src), file_name),
                         user=True,
                     )
-    
+
     def test_create_nbextensions_user(self):
         with TemporaryDirectory() as td:
             install_nbextension(self.src, user=True)
             self.assert_installed(
-                pjoin(basename(self.src), u'ƒile'),
+                pjoin(basename(self.src), 'ƒile'),
                 user=True
             )
-    
+
     def test_create_nbextensions_system(self):
         with TemporaryDirectory() as td:
-            self.system_nbext = pjoin(td, u'nbextensions')
+            self.system_nbext = pjoin(td, 'nbextensions')
             with patch.object(nbextensions, 'SYSTEM_JUPYTER_PATH', [td]):
                 install_nbextension(self.src, user=False)
                 self.assert_installed(
-                    pjoin(basename(self.src), u'ƒile'),
+                    pjoin(basename(self.src), 'ƒile'),
                     user=False
                 )
-    
+
     def test_single_file(self):
         file_name = self.files[0]
         install_nbextension(pjoin(self.src, file_name))
         self.assert_installed(file_name)
-    
+
     def test_single_dir(self):
-        d = u'∂ir'
+        d = '∂ir'
         install_nbextension(pjoin(self.src, d))
         self.assert_installed(self.files[-1])
-    
+
     def test_single_dir_trailing_slash(self):
-        d = u'∂ir/'
+        d = '∂ir/'
         install_nbextension(pjoin(self.src, d))
         self.assert_installed(self.files[-1])
         if os.name == 'nt':
-            d = u'∂ir\\'
+            d = '∂ir\\'
             install_nbextension(pjoin(self.src, d))
             self.assert_installed(self.files[-1])
 
     def test_destination_file(self):
         file_name = self.files[0]
-        install_nbextension(pjoin(self.src, file_name), destination = u'ƒiledest')
-        self.assert_installed(u'ƒiledest')
+        install_nbextension(pjoin(self.src, file_name), destination = 'ƒiledest')
+        self.assert_installed('ƒiledest')
 
     def test_destination_dir(self):
-        d = u'∂ir'
-        install_nbextension(pjoin(self.src, d), destination = u'ƒiledest2')
-        self.assert_installed(pjoin(u'ƒiledest2', u'∂ir2', u'ƒile2'))
-    
+        d = '∂ir'
+        install_nbextension(pjoin(self.src, d), destination = 'ƒiledest2')
+        self.assert_installed(pjoin('ƒiledest2', '∂ir2', 'ƒile2'))
+
     def test_install_nbextension(self):
         with self.assertRaises(TypeError):
             install_nbextension(glob.glob(pjoin(self.src, '*')))
-    
+
     def test_overwrite_file(self):
         with TemporaryDirectory() as d:
-            fname = u'ƒ.js'
+            fname = 'ƒ.js'
             src = pjoin(d, fname)
             with open(src, 'w') as f:
                 f.write('first')
@@ -216,24 +212,24 @@ class TestInstallNBExtension(TestCase):
             install_nbextension(src, overwrite=True)
             with open(dest) as f:
                 self.assertEqual(f.read(), 'overwrite')
-    
+
     def test_overwrite_dir(self):
         with TemporaryDirectory() as src:
             base = basename(src)
-            fname = u'ƒ.js'
+            fname = 'ƒ.js'
             touch(pjoin(src, fname))
             install_nbextension(src)
             self.assert_installed(pjoin(base, fname))
             os.remove(pjoin(src, fname))
-            fname2 = u'∂.js'
+            fname2 = '∂.js'
             touch(pjoin(src, fname2))
             install_nbextension(src, overwrite=True)
             self.assert_installed(pjoin(base, fname2))
             self.assert_not_installed(pjoin(base, fname))
-    
+
     def test_update_file(self):
         with TemporaryDirectory() as d:
-            fname = u'ƒ.js'
+            fname = 'ƒ.js'
             src = pjoin(d, fname)
             with open(src, 'w') as f:
                 f.write('first')
@@ -248,17 +244,17 @@ class TestInstallNBExtension(TestCase):
             install_nbextension(src)
             with open(dest) as f:
                 self.assertEqual(f.read(), 'overwrite')
-    
+
     def test_skip_old_file(self):
         with TemporaryDirectory() as d:
-            fname = u'ƒ.js'
+            fname = 'ƒ.js'
             src = pjoin(d, fname)
             mtime = touch(src)
             install_nbextension(src)
             self.assert_installed(fname)
             dest = pjoin(self.system_nbext, fname)
             old_mtime = os.stat(dest).st_mtime
-            
+
             mtime = touch(src, mtime - 100)
             install_nbextension(src)
             new_mtime = os.stat(dest).st_mtime
@@ -272,7 +268,7 @@ class TestInstallNBExtension(TestCase):
             install_nbextension(self.src)
         self.assertEqual(stdout.getvalue(), '')
         self.assertEqual(stderr.getvalue(), '')
-    
+
     def test_install_zip(self):
         path = pjoin(self.src, "myjsext.zip")
         with zipfile.ZipFile(path, 'w') as f:
@@ -281,22 +277,22 @@ class TestInstallNBExtension(TestCase):
         install_nbextension(path)
         self.assert_installed("a.js")
         self.assert_installed(pjoin("foo", "a.js"))
-    
+
     def test_install_tar(self):
         def _add_file(f, fname, buf):
             info = tarfile.TarInfo(fname)
             info.size = len(buf)
             f.addfile(info, BytesIO(buf))
-        
+
         for i,ext in enumerate((".tar.gz", ".tgz", ".tar.bz2")):
             path = pjoin(self.src, "myjsext" + ext)
             with tarfile.open(path, 'w') as f:
-                _add_file(f, "b%i.js" % i, b"b();")
-                _add_file(f, "foo/b%i.js" % i, b"foo();")
+                _add_file(f, f"b{i}.js", b"b();")
+                _add_file(f, f"foo/b{i}.js", b"foo();")
             install_nbextension(path)
-            self.assert_installed("b%i.js" % i)
-            self.assert_installed(pjoin("foo", "b%i.js" % i))
-    
+            self.assert_installed(f"b{i}.js")
+            self.assert_installed(pjoin("foo", f"b{i}.js"))
+
     def test_install_url(self):
         def fake_urlretrieve(url, dest):
             touch(dest)
@@ -307,27 +303,27 @@ class TestInstallNBExtension(TestCase):
             self.assert_installed("foo.js")
             install_nbextension("https://example.com/path/to/another/bar.js")
             self.assert_installed("bar.js")
-            install_nbextension("https://example.com/path/to/another/bar.js", 
+            install_nbextension("https://example.com/path/to/another/bar.js",
                                 destination = 'foobar.js')
             self.assert_installed("foobar.js")
         finally:
             nbextensions.urlretrieve = save_urlretrieve
-    
+
     def test_check_nbextension(self):
         with TemporaryDirectory() as d:
-            f = u'ƒ.js'
+            f = 'ƒ.js'
             src = pjoin(d, f)
             touch(src)
             install_nbextension(src, user=True)
-        
+
         assert check_nbextension(f, user=True)
         assert check_nbextension([f], user=True)
         assert not check_nbextension([f, pjoin('dne', f)], user=True)
-    
-    @dec.skip_win32
+
+    @pytest.mark.skipif(sys.platform == "win32", reason="do not run on windows")
     def test_install_symlink(self):
         with TemporaryDirectory() as d:
-            f = u'ƒ.js'
+            f = 'ƒ.js'
             src = pjoin(d, f)
             touch(src)
             install_nbextension(src, symlink=True)
@@ -335,12 +331,12 @@ class TestInstallNBExtension(TestCase):
         assert os.path.islink(dest)
         link = os.readlink(dest)
         self.assertEqual(link, src)
-    
-    @dec.skip_win32
+
+    @pytest.mark.skipif(sys.platform == "win32", reason="do not run on windows")
     def test_overwrite_broken_symlink(self):
         with TemporaryDirectory() as d:
-            f = u'ƒ.js'
-            f2 = u'ƒ2.js'
+            f = 'ƒ.js'
+            f2 = 'ƒ2.js'
             src = pjoin(d, f)
             src2 = pjoin(d, f2)
             touch(src)
@@ -352,11 +348,11 @@ class TestInstallNBExtension(TestCase):
         link = os.readlink(dest)
         self.assertEqual(link, src2)
 
-    @dec.skip_win32
+    @pytest.mark.skipif(sys.platform == "win32", reason="do not run on windows")
     def test_install_symlink_destination(self):
         with TemporaryDirectory() as d:
-            f = u'ƒ.js'
-            flink = u'ƒlink.js'
+            f = 'ƒ.js'
+            flink = 'ƒlink.js'
             src = pjoin(d, f)
             touch(src)
             install_nbextension(src, symlink=True, destination=flink)
@@ -365,13 +361,13 @@ class TestInstallNBExtension(TestCase):
         link = os.readlink(dest)
         self.assertEqual(link, src)
 
-    @dec.skip_win32
+    @pytest.mark.skipif(sys.platform == "win32", reason="do not run on windows")
     def test_install_symlink_bad(self):
         with self.assertRaises(ValueError):
             install_nbextension("http://example.com/foo.js", symlink=True)
 
         with TemporaryDirectory() as d:
-            zf = u'ƒ.zip'
+            zf = 'ƒ.zip'
             zsrc = pjoin(d, zf)
             with zipfile.ZipFile(zsrc, 'w') as z:
                 z.writestr("a.js", b"b();")
@@ -381,7 +377,7 @@ class TestInstallNBExtension(TestCase):
 
     def test_install_destination_bad(self):
         with TemporaryDirectory() as d:
-            zf = u'ƒ.zip'
+            zf = 'ƒ.zip'
             zsrc = pjoin(d, zf)
             with zipfile.ZipFile(zsrc, 'w') as z:
                 z.writestr("a.js", b"b();")
@@ -391,26 +387,26 @@ class TestInstallNBExtension(TestCase):
 
     def test_nbextension_enable(self):
         with TemporaryDirectory() as d:
-            f = u'ƒ.js'
+            f = 'ƒ.js'
             src = pjoin(d, f)
             touch(src)
             install_nbextension(src, user=True)
-            enable_nbextension(section='notebook', require=u'ƒ')
-        
+            enable_nbextension(section='notebook', require='ƒ')
+
         config_dir = os.path.join(_get_config_dir(user=True), 'nbconfig')
         cm = BaseJSONConfigManager(config_dir=config_dir)
-        enabled = cm.get('notebook').get('load_extensions', {}).get(u'ƒ', False)
+        enabled = cm.get('notebook').get('load_extensions', {}).get('ƒ', False)
         assert enabled
-    
+
     def test_nbextension_disable(self):
         self.test_nbextension_enable()
-        disable_nbextension(section='notebook', require=u'ƒ')
-        
+        disable_nbextension(section='notebook', require='ƒ')
+
         config_dir = os.path.join(_get_config_dir(user=True), 'nbconfig')
         cm = BaseJSONConfigManager(config_dir=config_dir)
-        enabled = cm.get('notebook').get('load_extensions', {}).get(u'ƒ', False)
+        enabled = cm.get('notebook').get('load_extensions', {}).get('ƒ', False)
         assert not enabled
-        
+
 
     def _mock_extension_spec_meta(self, section='notebook'):
         return {
@@ -427,52 +423,52 @@ class TestInstallNBExtension(TestCase):
 
         class mock():
             __file__ = outer_file
-            
+
             @staticmethod
             def _jupyter_nbextension_paths():
                 return [meta]
-        
+
         import sys
         sys.modules['mockextension'] = mock
-        
+
     def test_nbextensionpy_files(self):
         self._inject_mock_extension()
         install_nbextension_python('mockextension')
-        
+
         assert check_nbextension('_mockdestination/index.js')
         assert check_nbextension(['_mockdestination/index.js'])
-        
+
     def test_nbextensionpy_user_files(self):
         self._inject_mock_extension()
         install_nbextension_python('mockextension', user=True)
-        
+
         assert check_nbextension('_mockdestination/index.js', user=True)
         assert check_nbextension(['_mockdestination/index.js'], user=True)
-        
+
     def test_nbextensionpy_uninstall_files(self):
         self._inject_mock_extension()
         install_nbextension_python('mockextension', user=True)
         uninstall_nbextension_python('mockextension', user=True)
-        
+
         assert not check_nbextension('_mockdestination/index.js')
         assert not check_nbextension(['_mockdestination/index.js'])
-        
+
     def test_nbextensionpy_enable(self):
         self._inject_mock_extension('notebook')
         install_nbextension_python('mockextension', user=True)
         enable_nbextension_python('mockextension')
-        
+
         config_dir = os.path.join(_get_config_dir(user=True), 'nbconfig')
         cm = BaseJSONConfigManager(config_dir=config_dir)
         enabled = cm.get('notebook').get('load_extensions', {}).get('_mockdestination/index', False)
         assert enabled
-        
+
     def test_nbextensionpy_disable(self):
         self._inject_mock_extension('notebook')
         install_nbextension_python('mockextension', user=True)
         enable_nbextension_python('mockextension')
         disable_nbextension_python('mockextension', user=True)
-        
+
         config_dir = os.path.join(_get_config_dir(user=True), 'nbconfig')
         cm = BaseJSONConfigManager(config_dir=config_dir)
         enabled = cm.get('notebook').get('load_extensions', {}).get('_mockdestination/index', False)
